@@ -1,5 +1,5 @@
-# smf.rb: Written by Tadayoshi Funaba 1998-2001
-# $Id: smf.rb,v 1.25 2001-06-24 09:04:02+09 tadf Exp $
+# smf.rb: Written by Tadayoshi Funaba 1998-2002
+# $Id: smf.rb,v 1.26 2002-04-29 22:00:12+09 tadf Exp $
 
 module SMF
 
@@ -202,14 +202,29 @@ module SMF
 
       private :read_header, :read_meta, :read_track
 
+      def get_from_macbin
+	begin
+	  if @s[0] == 0 and @s[74] == 0 and @s[82] == 0 and @s[65,4] == 'Midi'
+	    @s[128,@s[83,4].unpack('N')[0]]
+	  end
+	rescue
+	end
+      end
+
+      def get_from_rmid
+	begin
+	  if @s[0,4] == 'RIFF' and @s[8,4] == 'RMID'
+	    @s[20,@s[16,4].unpack('V')[0]]
+	  end
+	rescue
+	end
+      end
+
+      private :get_from_macbin, :get_from_rmid
+
       def read
 	begin
-	  case @s[0,4]
-	  when 'RIFF'
-	    rs = RS::PO.new(@s[20,@s[16,4].unpack('V')[0]])
-	  else
-	    rs = RS::PO.new(@s)
-	  end
+	  rs = RS::PO.new(get_from_macbin || get_from_rmid || @s)
 	  ckid = rs.getn(4)
 	  unless ckid == 'MThd'
 	    @cb.error('not a SMF')
